/*
 * Caelum Manager
 * Author: Kencho
 * Starting date: 04-15-2004
 * Last update: 04-15-2004
 */
#include "CaelumManager.h"

namespace Ogre {
  //-------------------------------------------------------------------------------------
  CaelumManager::CaelumManager (SceneManager *sceneMgr, Camera *camera) {
    // Main object bindings
    setSceneManager (sceneMgr);
    setCamera (camera);

    // Create mandatory objects
    mDome = new CaelumDome (mSceneMgr, mSceneMgr->getRootSceneNode ());
    assert (mDome && "Couldn't create dome!!!");
    mHaze = new CaelumHaze ();
    assert (mHaze && "Couldn't create haze!!!");

    // Init attributes
    mTime = 0;
    mDay = 0;
    mLocation = Vector3 (0, 0, 0);
    mUnitScale = 1.0;
    mAxis = Vector3::UNIT_Y;
  }
  //-------------------------------------------------------------------------------------
  CaelumManager::~CaelumManager () {
    // Delete created objects
    if (mDome) {
      delete mDome;
    }
    if (mHaze) {
      delete mHaze;
    }
    ////// DELETE CAELUM OBJECTS......
  }
  //-------------------------------------------------------------------------------------
  void CaelumManager::setSceneManager (SceneManager *sceneMgr) {
    assert (sceneMgr && "SceneManager cannot be null!!!");
    mSceneMgr = sceneMgr;
  }
  //-------------------------------------------------------------------------------------
  SceneManager *CaelumManager::getSceneManager (void) {
    return mSceneMgr;
  }
  //-------------------------------------------------------------------------------------
  void CaelumManager::setCamera (Camera *camera) {
    assert (camera && "Camera cannot be null!!!");
    mCamera = camera;
  }
  //-------------------------------------------------------------------------------------
  Camera *CaelumManager::getCamera (void) {
    return mCamera;
  }
  //-------------------------------------------------------------------------------------
  CaelumDome *CaelumManager::getDome (void) {
    return mDome;
  }
  //-------------------------------------------------------------------------------------
  CaelumHaze *CaelumManager::getHaze (void) {
    return mHaze;
  }
  //-------------------------------------------------------------------------------------
  CaelumObject *CaelumManager::getObject (String name) {
    CaelumObjectList::iterator it;
    it = mObjectList.find (name);

    if (it != mObjectList.end ())
      return it->second;
    return 0;
  }
  //-------------------------------------------------------------------------------------
  CaelumObject *CaelumManager::createObject (String name, MovableObject *content) {
    // This will depend of the dome, so we will create a child from the dome node
    assert (mDome && mDome->getSceneNode () && "There must be a dome with a node to create objects");

//    CaelumObject *obj = new CaelumObject (name, mDome->getSceneNode ());
    CaelumObject *obj = new CaelumObject (name, mSceneMgr->getRootSceneNode ());
    SceneNode *node = obj->getSceneNode ();

    node->attachObject (content);
    content->setRenderQueueGroup (RENDER_QUEUE_2);

    //mObjectList.push_back (obj);
    mObjectList.insert (CaelumObjectList::value_type (name, obj));

    return obj;
  }
  //-------------------------------------------------------------------------------------
  void CaelumManager::setTime (Real time) {
    // Normalize time to 24 hrs ;)
    while (time >= 86400)
      time -= 86400;
    while (time < 0)
      time += 86400;

    mTime = time;
  }
  //-------------------------------------------------------------------------------------
  void CaelumManager::setTime (unsigned int hour, unsigned int minute, unsigned int second) {
    // Normalize times
    while (hour >= 24)
      hour -= 24;
    while (hour < 0)
      hour += 24;
    while (minute >= 60)
      minute -= 60;
    while (minute < 0)
      minute += 60;
    while (second >= 60)
      second -= 60;
    while (second < 0)
      second += 60;

    mTime = (hour * 3600) + (minute * 60) + (second);
  }
  //-------------------------------------------------------------------------------------
  Real CaelumManager::getTime (void) {
    return mTime;
  }
  //-------------------------------------------------------------------------------------
  void CaelumManager::setLocation (Vector3 location) {
    mLocation = location;
  }
  //-------------------------------------------------------------------------------------
  void CaelumManager::setLocation (Real longitude, Real latitude, Real radius) {
    mLocation.x = longitude;
    mLocation.y = latitude;
    mLocation.z = radius;
  }
  //-------------------------------------------------------------------------------------
  Vector3 CaelumManager::getLocation (void) {
    return mLocation;
  }
  //-------------------------------------------------------------------------------------
  void CaelumManager::setAxis (Vector3 axis) {
    mAxis = axis;
  }
  //-------------------------------------------------------------------------------------
  void CaelumManager::setAxis (Real x, Real y, Real z) {
    mAxis.x = x;
    mAxis.y = y;
    mAxis.z = z;
  }
  //-------------------------------------------------------------------------------------
  Vector3 CaelumManager::getAxis (void) {
    return mAxis;
  }
  //-------------------------------------------------------------------------------------
  void CaelumManager::update (Real time) {
    // Update time
    mTime += time;
    while (mTime >= 86400)
      mTime -= 86400;

    // Update Elements
    mHaze->update (mTime / 86400, mSceneMgr, mUnitScale);
    mDome->update (mTime / 86400, mCamera->getDerivedPosition (), mUnitScale / (mHaze->getTurbidity ()));
      // Update all the objects
    CaelumObjectList::iterator it;
    for (it = mObjectList.begin (); it != mObjectList.end (); it++) {
      it->second->update (mTime / 86400, mDome->getSceneNode ()->getPosition (), mDome->getSceneNode ()->getScale ());
    }

    mSceneMgr->setAmbientLight (mDome->getFinalColour ());
      // A small optimization ;)
//    mCamera->setFarClipDistance (mUnitScale / mHaze->getTurbidity ());
  }
  //-------------------------------------------------------------------------------------
}